@@ -4,7 +4,6 @@ import asyncio
 import socket
 import struct
 from time import perf_counter
-from typing import Optional
 
 import asyncio_dgram
 
@@ -12,7 +11,10 @@ from mcstatus.address import Address
 
 
 class BedrockServerStatus:
-    request_status_data = b"\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\x00\xfe\xfe\xfe\xfe\xfd\xfd\xfd\xfd\x124Vx"
+    request_status_data = bytes.fromhex(
+        # see https://wiki.vg/Raknet_Protocol#Unconnected_Ping
+        "01" + "000000000000000000" + "ffff00fefefefefdfdfdfd12345678" + "0000000000000000"  # fmt: skip
+    )
 
     def __init__(self, address: Address, timeout: float = 3):
         self.address = address
@@ -47,19 +49,28 @@ class BedrockServerStatus:
 
     def read_status(self) -> BedrockStatusResponse:
         start = perf_counter()
+        data = self._read_status()
+        end = perf_counter()
+        return self.parse_response(data, (end - start) * 1000)
 
+    def _read_status(self) -> bytes:
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         s.settimeout(self.timeout)
 
         s.sendto(self.request_status_data, self.address)
         data, _ = s.recvfrom(2048)
 
-        return self.parse_response(data, (perf_counter() - start))
+        return data
 
     async def read_status_async(self) -> BedrockStatusResponse:
         start = perf_counter()
-        stream = None
+        data = await self._read_status_async()
+        end = perf_counter()
 
+        return self.parse_response(data, (end - start) * 1000)
+
+    async def _read_status_async(self) -> bytes:
+        stream = None
         try:
             conn = asyncio_dgram.connect(self.address)
             stream = await asyncio.wait_for(conn, timeout=self.timeout)
@@ -70,7 +81,7 @@ class BedrockServerStatus:
             if stream is not None:
                 stream.close()
 
-        return self.parse_response(data, (perf_counter() - start))
+        return data
 
 
 class BedrockStatusResponse:
@@ -89,8 +100,8 @@ class BedrockStatusResponse:
         players_online: int,
         players_max: int,
         motd: str,
-        map_: Optional[str],
-        gamemode: Optional[str],
+        map_: str | None,
+        gamemode: str | None,
     ):
         self.version = self.Version(protocol, brand, version)
         self.latency = latency
