@@ -4,24 +4,25 @@ import asyncio
 import socket
 import struct
 from abc import ABC, abstractmethod
+from collections.abc import Iterable
 from ctypes import c_int32 as signed_int32
 from ctypes import c_int64 as signed_int64
 from ctypes import c_uint32 as unsigned_int32
 from ctypes import c_uint64 as unsigned_int64
 from ipaddress import ip_address
-from typing import Iterable, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING
 
 import asyncio_dgram
 
 from mcstatus.address import Address
 
 if TYPE_CHECKING:
-    from typing_extensions import SupportsIndex  # Python 3.7 doesn't support this yet.
+    from typing_extensions import Self, SupportsIndex, TypeAlias
 
-    BytesConvertable = Union[SupportsIndex, Iterable[SupportsIndex]]
+    BytesConvertable: TypeAlias = "SupportsIndex | Iterable[SupportsIndex]"
 
 
-def ip_type(address: Union[int, str]) -> Optional[int]:
+def ip_type(address: int | str) -> int | None:
     """Returns what version of IP a given address is."""
     try:
         return ip_address(address).version
@@ -35,7 +36,7 @@ class BaseWriteSync(ABC):
     __slots__ = ()
 
     @abstractmethod
-    def write(self, data: Union["Connection", str, bytearray, bytes]) -> None:
+    def write(self, data: Connection | str | bytearray | bytes) -> None:
         """Write data to self."""
         ...
 
@@ -131,7 +132,7 @@ class BaseWriteAsync(ABC):
     __slots__ = ()
 
     @abstractmethod
-    async def write(self, data: Union["Connection", str, bytearray, bytes]) -> None:
+    async def write(self, data: Connection | str | bytearray | bytes) -> None:
         """Write data to self."""
         ...
 
@@ -421,7 +422,7 @@ class BaseConnection:
         """Raise TypeError, unsupported."""
         raise TypeError(f"{self.__class__.__name__} does not support flush()")
 
-    def receive(self, data: Union[BytesConvertable, bytearray]) -> None:
+    def receive(self, data: BytesConvertable | bytearray) -> None:
         """Raise TypeError, unsupported."""
         raise TypeError(f"{self.__class__.__name__} does not support receive()")
 
@@ -464,7 +465,7 @@ class Connection(BaseSyncConnection):
         self.received = self.received[length:]
         return result
 
-    def write(self, data: Union["Connection", str, bytearray, bytes]) -> None:
+    def write(self, data: Connection | str | bytearray | bytes) -> None:
         """Extend self.sent from data."""
         if isinstance(data, Connection):
             data = data.flush()
@@ -472,7 +473,7 @@ class Connection(BaseSyncConnection):
             data = bytearray(data, "utf-8")
         self.sent.extend(data)
 
-    def receive(self, data: Union[BytesConvertable, bytearray]) -> None:
+    def receive(self, data: BytesConvertable | bytearray) -> None:
         """Extend self.received with data."""
         if not isinstance(data, bytearray):
             data = bytearray(data)
@@ -511,12 +512,11 @@ class SocketConnection(BaseSyncConnection):
             self.socket.shutdown(socket.SHUT_RDWR)
             self.socket.close()
 
-    def __del__(self) -> None:
-        """Close self.socket."""
-        try:
-            self.close()
-        except OSError:  # Probably, the socket was already closed by the OS
-            pass
+    def __enter__(self) -> Self:
+        return self
+
+    def __exit__(self, *_) -> None:
+        self.close()
 
 
 class TCPSocketConnection(SocketConnection):
@@ -524,7 +524,7 @@ class TCPSocketConnection(SocketConnection):
 
     __slots__ = ()
 
-    def __init__(self, addr: tuple[Optional[str], int], timeout: float = 3):
+    def __init__(self, addr: tuple[str | None, int], timeout: float = 3):
         """Create a connection to address with self.socket, set TCP NODELAY to True."""
         super().__init__()
         self.socket = socket.create_connection(addr, timeout=timeout)
@@ -540,7 +540,7 @@ class TCPSocketConnection(SocketConnection):
             result.extend(new)
         return result
 
-    def write(self, data: Union[Connection, str, bytes, bytearray]) -> None:
+    def write(self, data: Connection | str | bytes | bytearray) -> None:
         """Send data on self.socket."""
         if isinstance(data, Connection):
             data = bytearray(data.flush())
@@ -575,7 +575,7 @@ class UDPSocketConnection(SocketConnection):
             result.extend(self.socket.recvfrom(self.remaining())[0])
         return result
 
-    def write(self, data: Union[Connection, str, bytes, bytearray]) -> None:
+    def write(self, data: Connection | str | bytes | bytearray) -> None:
         """Use self.socket to send data to self.addr."""
         if isinstance(data, Connection):
             data = bytearray(data.flush())
@@ -587,18 +587,18 @@ class UDPSocketConnection(SocketConnection):
 class TCPAsyncSocketConnection(BaseAsyncReadSyncWriteConnection):
     """Asynchronous TCP Connection class"""
 
-    __slots__ = ("reader", "writer", "timeout")
+    __slots__ = ("reader", "writer", "timeout", "_addr")
 
-    def __init__(self) -> None:
+    def __init__(self, addr: Address, timeout: float = 3) -> None:
         # These will only be None until connect is called, ignore the None type assignment
         self.reader: asyncio.StreamReader = None  # type: ignore[assignment]
         self.writer: asyncio.StreamWriter = None  # type: ignore[assignment]
-        self.timeout: float = None  # type: ignore[assignment]
+        self.timeout: float = timeout
+        self._addr = addr
 
-    async def connect(self, addr: Address, timeout: float = 3) -> None:
+    async def connect(self) -> None:
         """Use asyncio to open a connection to address. Timeout is in seconds."""
-        self.timeout = timeout
-        conn = asyncio.open_connection(addr[0], addr[1])
+        conn = asyncio.open_connection(*self._addr)
         self.reader, self.writer = await asyncio.wait_for(conn, timeout=self.timeout)
 
     async def read(self, length: int) -> bytearray:
@@ -611,7 +611,7 @@ class TCPAsyncSocketConnection(BaseAsyncReadSyncWriteConnection):
             result.extend(new)
         return result
 
-    def write(self, data: Union[Connection, str, bytes, bytearray]) -> None:
+    def write(self, data: Connection | str | bytes | bytearray) -> None:
         """Write data to self.writer."""
         if isinstance(data, Connection):
             data = bytearray(data.flush())
@@ -622,30 +622,30 @@ class TCPAsyncSocketConnection(BaseAsyncReadSyncWriteConnection):
     def close(self) -> None:
         """Close self.writer."""
         if self.writer is not None:  # If initialized
-            try:
-                self.writer.close()
-            except RuntimeError:  # There is a case where event loop is closed
-                pass
+            self.writer.close()
 
-    def __del__(self) -> None:
-        """Close self."""
+    async def __aenter__(self) -> Self:
+        await self.connect()
+        return self
+
+    async def __aexit__(self, *_) -> None:
         self.close()
 
 
 class UDPAsyncSocketConnection(BaseAsyncConnection):
     """Asynchronous UDP Connection class"""
 
-    __slots__ = ("stream", "timeout")
+    __slots__ = ("stream", "timeout", "_addr")
 
-    def __init__(self) -> None:
+    def __init__(self, addr: Address, timeout: float = 3) -> None:
         # This will only be None until connect is called, ignore the None type assignment
         self.stream: asyncio_dgram.aio.DatagramClient = None  # type: ignore[assignment]
-        self.timeout: float = None  # type: ignore[assignment]
+        self.timeout: float = timeout
+        self._addr = addr
 
-    async def connect(self, addr: Address, timeout: float = 3) -> None:
+    async def connect(self) -> None:
         """Connect to address. Timeout is in seconds."""
-        self.timeout = timeout
-        conn = asyncio_dgram.connect((addr[0], addr[1]))
+        conn = asyncio_dgram.connect(self._addr)
         self.stream = await asyncio.wait_for(conn, timeout=self.timeout)
 
     def remaining(self) -> int:
@@ -657,7 +657,7 @@ class UDPAsyncSocketConnection(BaseAsyncConnection):
         data, remote_addr = await asyncio.wait_for(self.stream.recv(), timeout=self.timeout)
         return bytearray(data)
 
-    async def write(self, data: Union[Connection, str, bytes, bytearray]) -> None:
+    async def write(self, data: Connection | str | bytes | bytearray) -> None:
         """Send data with self.stream."""
         if isinstance(data, Connection):
             data = bytearray(data.flush())
@@ -670,9 +670,9 @@ class UDPAsyncSocketConnection(BaseAsyncConnection):
         if self.stream is not None:  # If initialized
             self.stream.close()
 
-    def __del__(self) -> None:
-        """Close self.stream"""
-        try:
-            self.close()
-        except (asyncio.exceptions.CancelledError, asyncio.exceptions.TimeoutError):
-            return
+    async def __aenter__(self) -> Self:
+        await self.connect()
+        return self
+
+    async def __aexit__(self, *_) -> None:
+        self.close()
