@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import warnings
+from abc import ABC
 from typing import TYPE_CHECKING
 
 import dns.resolver
@@ -15,57 +16,72 @@ from mcstatus.protocol.connection import (
     UDPSocketConnection,
 )
 from mcstatus.querier import AsyncServerQuerier, QueryResponse, ServerQuerier
-from mcstatus.utils import deprecated, retry
+from mcstatus.utils import retry
 
 if TYPE_CHECKING:
     from typing_extensions import Self
 
 
-__all__ = ["JavaServer", "BedrockServer", "MinecraftServer", "MinecraftBedrockServer"]
+__all__ = ["JavaServer", "BedrockServer"]
 
 
-class JavaServer:
-    """Base class for a Minecraft Java Edition server.
+class MCServer(ABC):
+    """Base abstract class for a general minecraft server.
 
-    :param str host: The host/address/ip of the Minecraft server.
+    This class only contains the basic logic shared across both java and bedrock versions,
+    it doesn't include any version specific settings and it can't be used to make any requests.
+
+    :param str host: The host/ip of the minecraft server.
     :param int port: The port that the server is on.
     :param float timeout: The timeout in seconds before failing to connect.
     """
 
-    def __init__(self, host: str, port: int = 25565, timeout: float = 3):
+    DEFAULT_PORT: int
+
+    def __init__(self, host: str, port: int | None = None, timeout: float = 3):
+        if port is None:
+            port = self.DEFAULT_PORT
         self.address = Address(host, port)
         self.timeout = timeout
 
-    @property
-    @deprecated(replacement="address.host", date="2022-08")
-    def host(self) -> str:
-        return self.address.host
+    @classmethod
+    def lookup(cls, address: str, timeout: float = 3) -> Self:
+        """Mimics minecraft's server address field.
 
-    @property
-    @deprecated(replacement="address.port", date="2022-08")
-    def port(self) -> int:
-        return self.address.port
+        :param str address: The address of the Minecraft server, like `example.com:19132`
+        :param float timeout: The timeout in seconds before failing to connect.
+        """
+        addr = Address.parse_address(address, default_port=cls.DEFAULT_PORT)
+        return cls(addr.host, addr.port, timeout=timeout)
+
+
+class JavaServer(MCServer):
+    """Base class for a Minecraft Java Edition server."""
+
+    DEFAULT_PORT = 25565
 
     @classmethod
     def lookup(cls, address: str, timeout: float = 3) -> Self:
-        """Parses the given address and checks DNS records for an SRV record that points to the Minecraft server.
+        """Mimics minecraft's server address field.
+
+        With Java servers, on top of just parsing the address, we also check the
+        DNS records for an SRV record that points to the server, which is the same
+        behavior as with minecraft's server address field for java. This DNS record
+        resolution is happening synchronously (see async_lookup).
 
         :param str address: The address of the Minecraft server, like `example.com:25565`.
         :param float timeout: The timeout in seconds before failing to connect.
-        :return: A `MinecraftServer` instance.
         """
-        addr = minecraft_srv_address_lookup(address, default_port=25565, lifetime=timeout)
+        addr = minecraft_srv_address_lookup(address, default_port=cls.DEFAULT_PORT, lifetime=timeout)
         return cls(addr.host, addr.port, timeout=timeout)
 
     @classmethod
     async def async_lookup(cls, address: str, timeout: float = 3) -> Self:
-        """Parses the given address and checks DNS records for an SRV record that points to the Minecraft server.
+        """Asynchronous alternative to lookup
 
-        :param str address: The address of the Minecraft server, like `example.com:25565`.
-        :param float timeout: The timeout in seconds before failing to connect.
-        :return: A `MinecraftServer` instance.
+        For more details, check the docstring of the synchronous lookup function.
         """
-        addr = await async_minecraft_srv_address_lookup(address, default_port=25565, lifetime=timeout)
+        addr = await async_minecraft_srv_address_lookup(address, default_port=cls.DEFAULT_PORT, lifetime=timeout)
         return cls(addr.host, addr.port, timeout=timeout)
 
     def ping(self, **kwargs) -> float:
@@ -73,11 +89,10 @@ class JavaServer:
 
         :param type **kwargs: Passed to a `ServerPinger` instance.
         :return: The latency between the Minecraft Server and you.
-        :rtype: float
         """
 
-        connection = TCPSocketConnection(self.address, self.timeout)
-        return self._retry_ping(connection, **kwargs)
+        with TCPSocketConnection(self.address, self.timeout) as connection:
+            return self._retry_ping(connection, **kwargs)
 
     @retry(tries=3)
     def _retry_ping(self, connection: TCPSocketConnection, **kwargs) -> float:
@@ -90,12 +105,10 @@ class JavaServer:
 
         :param type **kwargs: Passed to a `AsyncServerPinger` instance.
         :return: The latency between the Minecraft Server and you.
-        :rtype: float
         """
 
-        connection = TCPAsyncSocketConnection()
-        await connection.connect(self.address, self.timeout)
-        return await self._retry_async_ping(connection, **kwargs)
+        async with TCPAsyncSocketConnection(self.address, self.timeout) as connection:
+            return await self._retry_async_ping(connection, **kwargs)
 
     @retry(tries=3)
     async def _retry_async_ping(self, connection: TCPAsyncSocketConnection, **kwargs) -> float:
@@ -109,18 +122,16 @@ class JavaServer:
 
         :param type **kwargs: Passed to a `ServerPinger` instance.
         :return: Status information in a `PingResponse` instance.
-        :rtype: PingResponse
         """
 
-        connection = TCPSocketConnection(self.address, self.timeout)
-        return self._retry_status(connection, **kwargs)
+        with TCPSocketConnection(self.address, self.timeout) as connection:
+            return self._retry_status(connection, **kwargs)
 
     @retry(tries=3)
     def _retry_status(self, connection: TCPSocketConnection, **kwargs) -> PingResponse:
         pinger = ServerPinger(connection, address=self.address, **kwargs)
         pinger.handshake()
         result = pinger.read_status()
-        result.latency = pinger.test_ping()
         return result
 
     async def async_status(self, **kwargs) -> PingResponse:
@@ -128,27 +139,20 @@ class JavaServer:
 
         :param type **kwargs: Passed to a `AsyncServerPinger` instance.
         :return: Status information in a `PingResponse` instance.
-        :rtype: PingResponse
         """
 
-        connection = TCPAsyncSocketConnection()
-        await connection.connect(self.address, self.timeout)
-        return await self._retry_async_status(connection, **kwargs)
+        async with TCPAsyncSocketConnection(self.address, self.timeout) as connection:
+            return await self._retry_async_status(connection, **kwargs)
 
     @retry(tries=3)
     async def _retry_async_status(self, connection: TCPAsyncSocketConnection, **kwargs) -> PingResponse:
         pinger = AsyncServerPinger(connection, address=self.address, **kwargs)
         pinger.handshake()
         result = await pinger.read_status()
-        result.latency = await pinger.test_ping()
         return result
 
     def query(self) -> QueryResponse:
-        """Checks the status of a Minecraft Java Edition server via the query protocol.
-
-        :return: Query status information in a `QueryResponse` instance.
-        :rtype: QueryResponse
-        """
+        """Checks the status of a Minecraft Java Edition server via the query protocol."""
         # TODO: WARNING: This try-except for NXDOMAIN is only done because
         # of failing tests on mac-os, which for some reason can't resolve 'localhost'
         # into '127.0.0.1'. This try-except needs to be removed once this issue
@@ -163,17 +167,13 @@ class JavaServer:
 
     @retry(tries=3)
     def _retry_query(self, addr: Address) -> QueryResponse:
-        connection = UDPSocketConnection(addr, self.timeout)
-        querier = ServerQuerier(connection)
-        querier.handshake()
-        return querier.read_query()
+        with UDPSocketConnection(addr, self.timeout) as connection:
+            querier = ServerQuerier(connection)
+            querier.handshake()
+            return querier.read_query()
 
     async def async_query(self) -> QueryResponse:
-        """Asynchronously checks the status of a Minecraft Java Edition server via the query protocol.
-
-        :return: Query status information in a `QueryResponse` instance.
-        :rtype: QueryResponse
-        """
+        """Asynchronously checks the status of a Minecraft Java Edition server via the query protocol."""
         # TODO: WARNING: This try-except for NXDOMAIN is only done because
         # of failing tests on mac-os, which for some reason can't resolve 'localhost'
         # into '127.0.0.1'. This try-except needs to be removed once this issue
@@ -188,45 +188,16 @@ class JavaServer:
 
     @retry(tries=3)
     async def _retry_async_query(self, address: Address) -> QueryResponse:
-        connection = UDPAsyncSocketConnection()
-        await connection.connect(address, self.timeout)
-        querier = AsyncServerQuerier(connection)
-        await querier.handshake()
-        return await querier.read_query()
+        async with UDPAsyncSocketConnection(address, self.timeout) as connection:
+            querier = AsyncServerQuerier(connection)
+            await querier.handshake()
+            return await querier.read_query()
 
 
-class BedrockServer:
-    """Base class for a Minecraft Bedrock Edition server.
+class BedrockServer(MCServer):
+    """Base class for a Minecraft Bedrock Edition server."""
 
-    :param str host: The host/address/ip of the Minecraft server.
-    :param int port: The port that the server is on.
-    :param float timeout: The timeout in seconds before failing to connect.
-    """
-
-    def __init__(self, host: str, port: int = 19132, timeout: float = 3):
-        self.address = Address(host, port)
-        self.timeout = timeout
-
-    @property
-    @deprecated(replacement="address.host", date="2022-08")
-    def host(self) -> str:
-        return self.address.host
-
-    @property
-    @deprecated(replacement="address.host", date="2022-08")
-    def port(self) -> int:
-        return self.address.port
-
-    @classmethod
-    def lookup(cls, address: str, timeout: float = 3) -> Self:
-        """Parses a given address and returns a MinecraftBedrockServer instance.
-
-        :param str address: The address of the Minecraft server, like `example.com:19132`
-        :param float timeout: The timeout in seconds before failing to connect.
-        :return: A `MinecraftBedrockServer` instance.
-        """
-        addr = Address.parse_address(address, default_port=19132)
-        return cls(addr.host, addr.port, timeout=timeout)
+    DEFAULT_PORT = 19132
 
     @retry(tries=3)
     def status(self, **kwargs) -> BedrockStatusResponse:
@@ -247,25 +218,3 @@ class BedrockServer:
         :rtype: BedrockStatusResponse
         """
         return await BedrockServerStatus(self.address, self.timeout, **kwargs).read_status_async()
-
-
-@deprecated(replacement="JavaServer", date="2022-08", methods=("__init__",))
-class MinecraftServer(JavaServer):
-    """This is a deprecated version of the base class for a Java Minecraft Server.
-
-    This class is kept purely for backwards compatibility reasons and will be removed eventually.
-    """
-
-    def __init__(self, host: str, port: int = 25565, timeout: float = 3):
-        super().__init__(host, port=port, timeout=timeout)
-
-
-@deprecated(replacement="BedrockServer", date="2022-08", methods=("__init__",))
-class MinecraftBedrockServer(BedrockServer):
-    """This is a deprecated version of the base class for a Bedrock Minecraft Server.
-
-    This class is kept purely for backwards compatibility reasons and will be removed eventually.
-    """
-
-    def __init__(self, host: str, port: int = 19139, timeout: float = 3):
-        super().__init__(host, port=port, timeout=timeout)
